@@ -2,11 +2,11 @@
  * \file control/control_remap.c
  * \brief CTL Remap Plugin Interface
  * \author Jaroslav Kysela <perex@perex.cz>
- * \date 2021
+ * \date 2021-2025
  */
 /*
  *  Control - Remap Controls
- *  Copyright (c) 2021 by Jaroslav Kysela <perex@perex.cz>
+ *  Copyright (c) 2021-2025 by Jaroslav Kysela <perex@perex.cz>
  *
  *
  *   This library is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <stdarg.h>
 #include <unistd.h>
 #include <string.h>
@@ -76,28 +77,51 @@ typedef struct {
 		snd_ctl_elem_id_t id_child;
 		size_t channel_map_items;
 		size_t channel_map_alloc;
+		unsigned int src_channels;
 		long *channel_map;
 	} *controls;
-	unsigned int event_mask;
 } snd_ctl_map_t;
 
+typedef struct {
+	snd_ctl_elem_id_t *id;
+	unsigned int numid_app;
+	unsigned int event_mask;
+} snd_ctl_map_event_t;
+
+typedef struct {
+	size_t control_items;
+	snd_ctl_elem_id_t *control_ids;
+	snd_ctl_elem_id_t switch_id;
+	bool switch_state;
+} snd_ctl_sync_t;
+
 typedef struct {
 	snd_ctl_t *child;
 	int numid_remap_active;
 	unsigned int numid_app_last;
+
 	size_t numid_items;
 	size_t numid_alloc;
 	snd_ctl_numid_t *numid;
 	snd_ctl_numid_t numid_temp;
+
 	size_t remap_items;
 	size_t remap_alloc;
 	snd_ctl_remap_id_t *remap;
+
 	size_t map_items;
 	size_t map_alloc;
 	snd_ctl_map_t *map;
-	size_t map_read_queue_head;
-	size_t map_read_queue_tail;
-	snd_ctl_map_t **map_read_queue;
+
+	size_t sync_items;
+	size_t sync_alloc;
+	snd_ctl_sync_t *sync;
+	size_t sync_switch_items;
+
+	size_t event_items;
+	size_t event_queue_head;
+	size_t event_queue_tail;
+	snd_ctl_map_event_t *event_queue;
 } snd_ctl_remap_t;
 #endif
 
@@ -172,6 +196,23 @@ static snd_ctl_numid_t *remap_find_numid_child(snd_ctl_remap_t *priv, unsigned i
 	return remap_numid_child_new(priv, numid_child);
 }
 
+static void remap_forget_numid_child(snd_ctl_remap_t *priv, unsigned int numid_child)
+{
+	snd_ctl_numid_t *numid;
+	size_t index;
+
+	if (!priv->numid_remap_active)
+		return;
+	numid = priv->numid;
+	for (index = 0; index < priv->numid_items; index++) {
+		if (numid[index].numid_child != numid_child)
+			continue;
+		memcpy(&priv->numid[index], &priv->numid[index + 1],
+				(priv->numid_items - 1 - index) * sizeof(*numid));
+		priv->numid_items++;
+	}
+}
+
 static snd_ctl_remap_id_t *remap_find_id_child(snd_ctl_remap_t *priv, snd_ctl_elem_id_t *id)
 {
 	size_t count;
@@ -291,18 +332,102 @@ static int remap_id_to_app(snd_ctl_remap_t *priv, snd_ctl_elem_id_t *id, snd_ctl
 	return err;
 }
 
+static snd_ctl_sync_t *remap_find_sync_numid(snd_ctl_remap_t *priv, unsigned int numid)
+{
+	size_t count, index2;
+	snd_ctl_sync_t *sync;
+
+	if (numid == 0)
+		return NULL;
+	sync = priv->sync;
+	for (count = priv->sync_items; count > 0; count--, sync++)
+		for (index2 = 0; index2 < sync->control_items; index2++)
+			if (numid == sync->control_ids[index2].numid)
+				return sync;
+	return NULL;
+}
+
+static snd_ctl_sync_t *remap_find_sync_id(snd_ctl_remap_t *priv, snd_ctl_elem_id_t *id)
+{
+	size_t count, index2;
+	snd_ctl_sync_t *sync;
+
+	if (id->numid > 0)
+		return remap_find_sync_numid(priv, id->numid);
+	sync = priv->sync;
+	for (count = priv->sync_items; count > 0; count--, sync++)
+		for (index2 = 0; index2 < sync->control_items; index2++)
+			if (snd_ctl_elem_id_compare_set(id, &sync->control_ids[index2]) == 0)
+				return sync;
+	return NULL;
+}
+
+static void remap_update_sync_id(snd_ctl_remap_t *priv, snd_ctl_elem_id_t *id)
+{
+	size_t count, index2;
+	snd_ctl_sync_t *sync;
+
+	if (id->numid == 0)
+		return;
+	sync = priv->sync;
+	for (count = priv->sync_items; count > 0; count--, sync++)
+		for (index2 = 0; index2 < sync->control_items; index2++) {
+			if (snd_ctl_elem_id_compare_set(id, &sync->control_ids[index2]) == 0) {
+				sync->control_ids[index2].numid = id->numid;
+				break;
+			}
+		}
+}
+
+static snd_ctl_sync_t *remap_find_sync_switch_numid(snd_ctl_remap_t *priv, unsigned int numid)
+{
+	size_t count;
+	snd_ctl_sync_t *sync;
+
+	if (numid == 0)
+		return NULL;
+	sync = priv->sync;
+	for (count = priv->sync_items; count > 0; count--, sync++)
+		if (numid == sync->switch_id.numid)
+			return sync;
+	return NULL;
+}
+
+static snd_ctl_sync_t *remap_find_sync_switch_id(snd_ctl_remap_t *priv, snd_ctl_elem_id_t *id)
+{
+	size_t count;
+	snd_ctl_sync_t *sync;
+
+	if (id->numid > 0)
+		return remap_find_sync_switch_numid(priv, id->numid);
+	sync = priv->sync;
+	for (count = priv->sync_items; count > 0; count--, sync++) {
+		if (sync->switch_id.numid == 0)
+			continue;
+		if (snd_ctl_elem_id_compare_set(id, &sync->switch_id) == 0)
+			return sync;
+	}
+	return NULL;
+}
+
 static void remap_free(snd_ctl_remap_t *priv)
 {
 	size_t idx1, idx2;
+	snd_ctl_sync_t *sync;
 	snd_ctl_map_t *map;
 
+	for (idx1 = 0; idx1 < priv->sync_items; idx1++) {
+		sync = &priv->sync[idx1];
+		free(sync->control_ids);
+	}
 	for (idx1 = 0; idx1 < priv->map_items; idx1++) {
 		map = &priv->map[idx1];
 		for (idx2 = 0; idx2 < map->controls_items; idx2++)
 			free(map->controls[idx2].channel_map);
 		free(map->controls);
 	}
-	free(priv->map_read_queue);
+	free(priv->event_queue);
+	free(priv->sync);
 	free(priv->map);
 	free(priv->remap);
 	free(priv->numid);
@@ -347,7 +472,6 @@ static int snd_ctl_remap_elem_list(snd_ctl_t *ctl, snd_ctl_elem_list_t *list)
 	snd_ctl_elem_id_t *id;
 	snd_ctl_remap_id_t *rid;
 	snd_ctl_numid_t *numid;
-	snd_ctl_map_t *map;
 	unsigned int index;
 	size_t index2;
 	int err;
@@ -367,18 +491,25 @@ static int snd_ctl_remap_elem_list(snd_ctl_t *ctl, snd_ctl_elem_list_t *list)
 			return -EIO;
 		id->numid = numid->numid_app;
 	}
-	if (list->offset >= list->count + priv->map_items)
+	if (list->offset >= list->count + priv->map_items + priv->sync_switch_items)
 		return 0;
 	index2 = 0;
 	if (list->offset > list->count)
 		index2 = list->offset - list->count;
 	for ( ; index < list->space && index2 < priv->map_items; index2++, index++) {
-		id = &list->pids[index];
-		map = &priv->map[index2];
-		*id = map->map_id;
+		snd_ctl_map_t *map = &priv->map[index2];
+		list->pids[index] = map->map_id;
 		list->used++;
 	}
-	list->count += priv->map_items;
+	if (index2 >= priv->map_items) {
+		index2 -= priv->map_items;
+		for ( ; index < list->space && index2 < priv->sync_switch_items; index2++, index++) {
+			snd_ctl_sync_t *sync = &priv->sync[index2];
+			list->pids[index] = sync->switch_id;
+			list->used++;
+		}
+	}
+	list->count += priv->map_items + priv->sync_switch_items;
 	return 0;
 }
 
@@ -459,6 +590,21 @@ static int remap_map_elem_info(snd_ctl_remap_t *priv, snd_ctl_elem_info_t *info)
 	return 0;
 }
 
+static int remap_sync_elem_info(snd_ctl_remap_t *priv, snd_ctl_elem_info_t *info)
+{
+	snd_ctl_sync_t *sync;
+
+	sync = remap_find_sync_switch_id(priv, &info->id);
+	if (!sync)
+		return -EREMAPNOTFOUND;
+	snd_ctl_elem_info_clear(info);
+	info->id = sync->switch_id;
+	info->type = SNDRV_CTL_ELEM_TYPE_BOOLEAN;
+	info->access = SNDRV_CTL_ELEM_ACCESS_READ | SNDRV_CTL_ELEM_ACCESS_WRITE;
+	info->count = 1;
+	return 0;
+}
+
 static int snd_ctl_remap_elem_info(snd_ctl_t *ctl, snd_ctl_elem_info_t *info)
 {
 	snd_ctl_remap_t *priv = ctl->private_data;
@@ -467,12 +613,17 @@ static int snd_ctl_remap_elem_info(snd_ctl_t *ctl, snd_ctl_elem_info_t *info)
 
 	debug_id(&info->id, "%s\n", __func__);
 	err = remap_map_elem_info(priv, info);
+	if (err != -EREMAPNOTFOUND)
+		return err;
+	err = remap_sync_elem_info(priv, info);
 	if (err != -EREMAPNOTFOUND)
 		return err;
 	err = remap_id_to_child(priv, &info->id, &rid);
 	if (err < 0)
 		return err;
 	err = snd_ctl_elem_info(priv->child, info);
+	if (err >= 0 && priv->sync_items > 0)
+		remap_update_sync_id(priv, &info->id);
 	return remap_id_to_app(priv, &info->id, rid, err);
 }
 
@@ -481,7 +632,7 @@ static int remap_map_elem_read(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *cont
 	snd_ctl_map_t *map;
 	struct snd_ctl_map_ctl *mctl;
 	snd_ctl_elem_value_t control2;
-	size_t item, index;
+	size_t item, index, src_index;
 	int err;
 
 	map = remap_find_map_id(priv, &control->id);
@@ -501,19 +652,36 @@ static int remap_map_elem_read(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *cont
 		if (map->type == SNDRV_CTL_ELEM_TYPE_BOOLEAN ||
 		    map->type == SNDRV_CTL_ELEM_TYPE_INTEGER) {
 			for (index = 0; index < mctl->channel_map_items; index++) {
-				long src = mctl->channel_map[index];
+				long base_idx = mctl->src_channels * index;
+				long src = mctl->channel_map[base_idx];
 				if ((unsigned long)src < ARRAY_SIZE(control->value.integer.value))
 					control->value.integer.value[index] = control2.value.integer.value[src];
+				for (src_index = 1; src_index < mctl->src_channels; src_index++) {
+					src = mctl->channel_map[base_idx + src_index];
+					if ((unsigned long)src < ARRAY_SIZE(control->value.integer.value))
+						if (control2.value.integer.value[src] < control->value.integer.value[index])
+							control->value.integer.value[index] = control2.value.integer.value[src];
+				}
 			}
 		} else if (map->type == SNDRV_CTL_ELEM_TYPE_INTEGER64) {
 			for (index = 0; index < mctl->channel_map_items; index++) {
-				long src = mctl->channel_map[index];
+				long base_idx = mctl->src_channels * index;
+				long src = mctl->channel_map[base_idx];
 				if ((unsigned long)src < ARRAY_SIZE(control->value.integer64.value))
 					control->value.integer64.value[index] = control2.value.integer64.value[src];
+				for (src_index = 1; src_index < mctl->src_channels; src_index++) {
+					src = mctl->channel_map[base_idx + src_index];
+					if ((unsigned long)src < ARRAY_SIZE(control->value.integer64.value))
+						if (control2.value.integer64.value[src] < control->value.integer64.value[index])
+							control->value.integer64.value[index] = control2.value.integer64.value[src];
+				}
 			}
 		} else if (map->type == SNDRV_CTL_ELEM_TYPE_BYTES) {
+			/* does it make sense to merge bytes? */
+			if (mctl->src_channels > 1)
+				return -EINVAL;
 			for (index = 0; index < mctl->channel_map_items; index++) {
-				long src = mctl->channel_map[index];
+				long src = mctl->channel_map[mctl->src_channels];
 				if ((unsigned long)src < ARRAY_SIZE(control->value.bytes.data))
 					control->value.bytes.data[index] = control2.value.bytes.data[src];
 			}
@@ -522,6 +690,17 @@ static int remap_map_elem_read(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *cont
 	return 0;
 }
 
+static int remap_sync_elem_read(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *control)
+{
+	snd_ctl_sync_t *sync;
+
+	sync = remap_find_sync_switch_id(priv, &control->id);
+	if (!sync)
+		return -EREMAPNOTFOUND;
+	control->value.integer.value[0] = sync->switch_state ? 1 : 0;
+	return 0;
+}
+
 static int snd_ctl_remap_elem_read(snd_ctl_t *ctl, snd_ctl_elem_value_t *control)
 {
 	snd_ctl_remap_t *priv = ctl->private_data;
@@ -530,6 +709,9 @@ static int snd_ctl_remap_elem_read(snd_ctl_t *ctl, snd_ctl_elem_value_t *control
 
 	debug_id(&control->id, "%s\n", __func__);
 	err = remap_map_elem_read(priv, control);
+	if (err != -EREMAPNOTFOUND)
+		return err;
+	err = remap_sync_elem_read(priv, control);
 	if (err != -EREMAPNOTFOUND)
 		return err;
 	err = remap_id_to_child(priv, &control->id, &rid);
@@ -544,7 +726,7 @@ static int remap_map_elem_write(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *con
 	snd_ctl_map_t *map;
 	struct snd_ctl_map_ctl *mctl;
 	snd_ctl_elem_value_t control2;
-	size_t item, index;
+	size_t item, index, src_index;
 	int err, changes;
 
 	map = remap_find_map_id(priv, &control->id);
@@ -564,21 +746,40 @@ static int remap_map_elem_write(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *con
 		if (map->type == SNDRV_CTL_ELEM_TYPE_BOOLEAN ||
 		    map->type == SNDRV_CTL_ELEM_TYPE_INTEGER) {
 			for (index = 0; index < mctl->channel_map_items; index++) {
-				long dst = mctl->channel_map[index];
+				long base_idx = mctl->src_channels * index;
+				long dst = mctl->channel_map[base_idx];
 				if ((unsigned long)dst < ARRAY_SIZE(control->value.integer.value)) {
 					changes |= control2.value.integer.value[dst] != control->value.integer.value[index];
 					control2.value.integer.value[dst] = control->value.integer.value[index];
 				}
+				for (src_index = 1; src_index < mctl->src_channels; src_index++) {
+					dst = mctl->channel_map[base_idx + src_index];
+					if ((unsigned long)dst < ARRAY_SIZE(control->value.integer.value)) {
+						changes |= control2.value.integer.value[dst] != control->value.integer.value[index];
+						control2.value.integer.value[dst] = control->value.integer.value[index];
+					}
+				}
 			}
 		} else if (map->type == SNDRV_CTL_ELEM_TYPE_INTEGER64) {
 			for (index = 0; index < mctl->channel_map_items; index++) {
-				long dst = mctl->channel_map[index];
+				long base_idx = mctl->src_channels * index;
+				long dst = mctl->channel_map[base_idx];
 				if ((unsigned long)dst < ARRAY_SIZE(control->value.integer64.value)) {
 					changes |= control2.value.integer64.value[dst] != control->value.integer64.value[index];
 					control2.value.integer64.value[dst] = control->value.integer64.value[index];
 				}
+				for (src_index = 1; src_index < mctl->src_channels; src_index++) {
+					dst = mctl->channel_map[base_idx + src_index];
+					if ((unsigned long)dst < ARRAY_SIZE(control->value.integer.value)) {
+						changes |= control2.value.integer64.value[dst] != control->value.integer64.value[index];
+						control2.value.integer64.value[dst] = control->value.integer64.value[index];
+					}
+				}
 			}
 		} else if (map->type == SNDRV_CTL_ELEM_TYPE_BYTES) {
+			/* does it make sense to merge bytes? */
+			if (mctl->src_channels > 1)
+				return -EINVAL;
 			for (index = 0; index < mctl->channel_map_items; index++) {
 				long dst = mctl->channel_map[index];
 				if ((unsigned long)dst < ARRAY_SIZE(control->value.bytes.data)) {
@@ -597,6 +798,37 @@ static int remap_map_elem_write(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *con
 	return 0;
 }
 
+static int remap_sync_elem_write(snd_ctl_remap_t *priv, snd_ctl_elem_value_t *control)
+{
+	snd_ctl_sync_t *sync;
+	snd_ctl_elem_value_t control2;
+	size_t item;
+	int err;
+
+	sync = remap_find_sync_switch_id(priv, &control->id);
+	if (sync) {
+		err = sync->switch_state != control->value.integer.value[0];
+		sync->switch_state = control->value.integer.value[0] != 0;
+		return err;
+	}
+	sync = remap_find_sync_id(priv, &control->id);
+	if (sync == NULL)
+		return -EREMAPNOTFOUND;
+	if (sync->switch_state == false)
+		return -EREMAPNOTFOUND;
+	debug_id(&control->id, "%s\n", __func__);
+	control2 = *control;
+	for (item = 0; item < sync->control_items; item++) {
+		control2.id = sync->control_ids[item];
+		debug_id(&control2.id, "%s sync[%zd]\n", __func__, item);
+		/* TODO: it's a blind write - no checks if the values are in range for all controls */
+		err = snd_ctl_elem_write(priv->child, &control2);
+		if (err < 0)
+			return err;
+	}
+	return remap_id_to_app(priv, &control->id, NULL, 0);
+}
+
 static int snd_ctl_remap_elem_write(snd_ctl_t *ctl, snd_ctl_elem_value_t *control)
 {
 	snd_ctl_remap_t *priv = ctl->private_data;
@@ -605,6 +837,9 @@ static int snd_ctl_remap_elem_write(snd_ctl_t *ctl, snd_ctl_elem_value_t *contro
 
 	debug_id(&control->id, "%s\n", __func__);
 	err = remap_map_elem_write(priv, control);
+	if (err != -EREMAPNOTFOUND)
+		return err;
+	err = remap_sync_elem_write(priv, control);
 	if (err != -EREMAPNOTFOUND)
 		return err;
 	err = remap_id_to_child(priv, &control->id, &rid);
@@ -794,19 +1029,45 @@ static void _next_ptr(size_t *ptr, size_t count)
 	*ptr = (*ptr + 1) % count;
 }
 
+static void event_add(snd_ctl_remap_t *priv, snd_ctl_elem_id_t *id,
+		      unsigned int numid_app, unsigned int event_mask)
+{
+	snd_ctl_map_event_t *map_event;
+	int found = 0;
+	size_t head;
+
+	for (head = priv->event_queue_head;
+	     head != priv->event_queue_tail;
+	     _next_ptr(&head, priv->event_items))
+		if (priv->event_queue[head].numid_app == numid_app) {
+			found = 1;
+			priv->event_queue[head].event_mask |= event_mask;
+			break;
+		}
+	debug_id(id, "%s marking for read (already %d)\n", __func__, found);
+	if (found)
+		return;
+	map_event = &priv->event_queue[priv->event_queue_tail];
+	map_event->id = id;
+	map_event->numid_app = numid_app;
+	map_event->event_mask = event_mask;
+	_next_ptr(&priv->event_queue_tail, priv->event_items);
+}
+
 static void remap_event_for_all_map_controls(snd_ctl_remap_t *priv,
 					     snd_ctl_elem_id_t *id,
 					     unsigned int event_mask)
 {
-	size_t count, index, head;
+	size_t count, index;
 	snd_ctl_map_t *map;
 	struct snd_ctl_map_ctl *mctl;
-	int found;
+	int changed = 0;
 
 	if (event_mask == SNDRV_CTL_EVENT_MASK_REMOVE)
 		event_mask = SNDRV_CTL_EVENT_MASK_INFO;
 	map = priv->map;
 	for (count = priv->map_items; count > 0; count--, map++) {
+		changed = 0;
 		for (index = 0; index < map->controls_items; index++) {
 			mctl = &map->controls[index];
 			if (mctl->id_child.numid == 0) {
@@ -817,20 +1078,51 @@ static void remap_event_for_all_map_controls(snd_ctl_remap_t *priv,
 			if (id->numid != mctl->id_child.numid)
 				continue;
 			debug_id(&map->map_id, "%s found (all)\n", __func__);
-			map->event_mask |= event_mask;
-			found = 0;
-			for (head = priv->map_read_queue_head;
-			     head != priv->map_read_queue_tail;
-			     _next_ptr(&head, priv->map_items))
-				if (priv->map_read_queue[head] == map) {
-					found = 1;
-					break;
-				}
-			if (found)
+			changed = 1;
+		}
+		if (changed)
+			event_add(priv, &map->map_id, map->map_id.numid, event_mask);
+	}
+}
+
+static void remap_event_for_all_sync_controls(snd_ctl_remap_t *priv,
+					      snd_ctl_elem_id_t *id,
+					      unsigned int event_mask)
+{
+	size_t count, index;
+	snd_ctl_sync_t *sync;
+	snd_ctl_numid_t *numid;
+	snd_ctl_elem_id_t *sid;
+	int changed = 0;
+
+	if (event_mask == SNDRV_CTL_EVENT_MASK_REMOVE)
+		return;
+	sync = priv->sync;
+	for (count = priv->sync_items; count > 0; count--, sync++) {
+		changed = 0;
+		for (index = 0; index < sync->control_items; index++) {
+			sid = &sync->control_ids[index];
+			if (sid->numid == 0) {
+				if (snd_ctl_elem_id_compare_set(id, sid))
+					continue;
+				sid->numid = id->numid;
+			}
+			if (id->numid != sid->numid)
 				continue;
-			debug_id(&map->map_id, "%s marking for read\n", __func__);
-			priv->map_read_queue[priv->map_read_queue_tail] = map;
-			_next_ptr(&priv->map_read_queue_tail, priv->map_items);
+			debug_id(sid, "%s found (all)\n", __func__);
+			changed = 1;
+			break;
+		}
+		if (!changed)
+			continue;
+		for (index = 0; index < sync->control_items; index++) {
+			sid = &sync->control_ids[index];
+			/* skip double updates */
+			if (sid->numid == id->numid)
+				continue;
+			numid = remap_find_numid_child(priv, sid->numid);
+			if (numid)
+				event_add(priv, sid, numid->numid_app, event_mask);
 		}
 	}
 }
@@ -840,18 +1132,19 @@ static int snd_ctl_remap_read(snd_ctl_t *ctl, snd_ctl_event_t *event)
 	snd_ctl_remap_t *priv = ctl->private_data;
 	snd_ctl_remap_id_t *rid;
 	snd_ctl_numid_t *numid;
-	snd_ctl_map_t *map;
+	snd_ctl_map_event_t *map_event;
+	unsigned int numid_child;
 	int err;
 
-	if (priv->map_read_queue_head != priv->map_read_queue_tail) {
-		map = priv->map_read_queue[priv->map_read_queue_head];
-		_next_ptr(&priv->map_read_queue_head, priv->map_items);
+	if (priv->event_queue_head != priv->event_queue_tail) {
+		map_event = &priv->event_queue[priv->event_queue_head];
+		_next_ptr(&priv->event_queue_head, priv->event_items);
 		memset(event, 0, sizeof(*event));
 		event->type = SNDRV_CTL_EVENT_ELEM;
-		event->data.elem.mask = map->event_mask;
-		event->data.elem.id = map->map_id;
-		map->event_mask = 0;
-		debug_id(&map->map_id, "%s queue read\n", __func__);
+		event->data.elem.mask = map_event->event_mask;
+		event->data.elem.id = *map_event->id;
+		event->data.elem.id.numid = map_event->numid_app;
+		debug_id(&event->data.elem.id, "%s queue read\n", __func__);
 		return 1;
 	}
 	err = snd_ctl_read(priv->child, event);
@@ -861,11 +1154,13 @@ static int snd_ctl_remap_read(snd_ctl_t *ctl, snd_ctl_event_t *event)
 	    (event->data.elem.mask & (SNDRV_CTL_EVENT_MASK_VALUE | SNDRV_CTL_EVENT_MASK_INFO |
 				      SNDRV_CTL_EVENT_MASK_ADD | SNDRV_CTL_EVENT_MASK_TLV)) != 0) {
 		debug_id(&event->data.elem.id, "%s event mask 0x%x\n", __func__, event->data.elem.mask);
+		numid_child = event->data.elem.id.numid;
 		remap_event_for_all_map_controls(priv, &event->data.elem.id, event->data.elem.mask);
+		remap_event_for_all_sync_controls(priv, &event->data.elem.id, event->data.elem.mask);
 		rid = remap_find_id_child(priv, &event->data.elem.id);
 		if (rid) {
 			if (rid->id_child.numid == 0) {
-				numid = remap_find_numid_child(priv, event->data.elem.id.numid);
+				numid = remap_find_numid_child(priv, numid_child);
 				if (numid == NULL)
 					return -EIO;
 				rid->id_child.numid = numid->numid_child;
@@ -873,11 +1168,13 @@ static int snd_ctl_remap_read(snd_ctl_t *ctl, snd_ctl_event_t *event)
 			}
 			event->data.elem.id = rid->id_app;
 		} else {
-			numid = remap_find_numid_child(priv, event->data.elem.id.numid);
+			numid = remap_find_numid_child(priv, numid_child);
 			if (numid == NULL)
 				return -EIO;
 			event->data.elem.id.numid = numid->numid_app;
 		}
+		if (event->data.elem.mask == SNDRV_CTL_EVENT_MASK_REMOVE)
+			remap_forget_numid_child(priv, numid_child);
 	}
 	return err;
 }
@@ -914,6 +1211,7 @@ static int add_to_remap(snd_ctl_remap_t *priv,
 {
 	snd_ctl_remap_id_t *rid;
 
+
 	if (priv->remap_alloc == priv->remap_items) {
 		rid = realloc(priv->remap, (priv->remap_alloc + 16) * sizeof(*rid));
 		if (rid == NULL)
@@ -1017,43 +1315,79 @@ static int add_ctl_to_map(snd_ctl_map_t *map, struct snd_ctl_map_ctl **_mctl, sn
 	return 0;
 }
 
-static int add_chn_to_map(struct snd_ctl_map_ctl *mctl, long idx, long val)
+static int add_chn_to_map(struct snd_ctl_map_ctl *mctl, long idx, long src_idx, long val)
 {
 	size_t off;
 	long *map;
 
+	if (src_idx >= mctl->src_channels) {
+		SNDERR("Wrong channel mapping (extra source channel?)");
+		return -EINVAL;
+	}
 	if (mctl->channel_map_alloc <= (size_t)idx) {
-		map = realloc(mctl->channel_map, (idx + 4) * sizeof(*map));
+		map = realloc(mctl->channel_map, mctl->src_channels * (idx + 4) * sizeof(*map));
 		if (map == NULL)
 			return -ENOMEM;
 		mctl->channel_map = map;
 		off = mctl->channel_map_alloc;
 		mctl->channel_map_alloc = idx + 4;
-		for ( ; off < mctl->channel_map_alloc; off++)
+		for ( ; off < mctl->src_channels * mctl->channel_map_alloc; off++)
 			map[off] = -1;
 	}
 	if ((size_t)idx >= mctl->channel_map_items)
 		mctl->channel_map_items = idx + 1;
-	mctl->channel_map[idx] = val;
+	mctl->channel_map[mctl->src_channels * idx + src_idx] = val;
+	return 0;
+}
+
+static int add_chn_to_map_array(struct snd_ctl_map_ctl *mctl, const char *dst_id, snd_config_t *conf)
+{
+	snd_config_iterator_t i, next;
+	long src_idx = 0;
+	int err;
+
+	snd_config_for_each(i, next, conf) {
+		snd_config_t *n = snd_config_iterator_entry(i);
+		long idx = -1, chn = -1;
+		if (safe_strtol(dst_id, &idx) || snd_config_get_integer(n, &chn)) {
+			SNDERR("Wrong channel mapping (%ld -> %ld)", idx, chn);
+			return -EINVAL;
+		}
+		err = add_chn_to_map(mctl, idx, src_idx, chn);
+		if (err < 0)
+			return err;
+		src_idx++;
+	}
 	return 0;
 }
 
 static int parse_map_vindex(struct snd_ctl_map_ctl *mctl, snd_config_t *conf)
 {
 	snd_config_iterator_t i, next;
+	snd_config_t *n;
 	int err;
 
 	snd_config_for_each(i, next, conf) {
-		snd_config_t *n = snd_config_iterator_entry(i);
+		n = snd_config_iterator_entry(i);
+		err = snd_config_is_array(n);
+		if (err > 0 && (unsigned int)err > mctl->src_channels)
+			mctl->src_channels = err; /* count of array items */
+	}
+	snd_config_for_each(i, next, conf) {
+		n = snd_config_iterator_entry(i);
 		long idx = -1, chn = -1;
 		const char *id;
 		if (snd_config_get_id(n, &id) < 0)
 			continue;
-		if (safe_strtol(id, &idx) || snd_config_get_integer(n, &chn)) {
-			SNDERR("Wrong channel mapping (%ld -> %ld)", idx, chn);
-			return -EINVAL;
+		if (snd_config_is_array(n) > 0) {
+			err = add_chn_to_map_array(mctl, id, n);
+		} else {
+			if (safe_strtol(id, &idx) || snd_config_get_integer(n, &chn)) {
+				SNDERR("Wrong channel mapping (%ld -> %ld)", idx, chn);
+				return -EINVAL;
+			}
+			err = add_chn_to_map(mctl, idx, 0, chn);
 		}
-		err = add_chn_to_map(mctl, idx, chn);
 		if (err < 0)
 			return err;
 	}
@@ -1066,6 +1400,7 @@ static int parse_map_config(struct snd_ctl_map_ctl *mctl, snd_config_t *conf)
 	snd_config_iterator_t i, next;
 	int err;
 
+	mctl->src_channels = 1;
 	snd_config_for_each(i, next, conf) {
 		snd_config_t *n = snd_config_iterator_entry(i);
 		const char *id;
@@ -1140,12 +1475,141 @@ static int parse_map(snd_ctl_remap_t *priv, snd_config_t *conf)
 	return 0;
 }
 
+static snd_ctl_sync_t *alloc_sync(snd_ctl_remap_t *priv)
+{
+	snd_ctl_sync_t *sync;
+
+	if (priv->sync_alloc == priv->sync_items) {
+		sync = realloc(priv->sync, (priv->sync_alloc + 16) * sizeof(*sync));
+		if (sync == NULL)
+			return NULL;
+		memset(sync + priv->sync_alloc, 0, sizeof(*sync) * 16);
+		priv->sync_alloc += 16;
+		priv->sync = sync;
+	}
+	return &priv->sync[priv->sync_items++];
+}
+
+static int parse_sync1(snd_ctl_remap_t *priv, unsigned int count, snd_config_t *conf)
+{
+	snd_config_iterator_t i, next;
+	snd_ctl_elem_id_t *eid;
+	snd_ctl_sync_t *sync;
+	const char *str;
+	int err, index = 0;
+
+	sync = alloc_sync(priv);
+	if (sync == NULL)
+		return -ENOMEM;
+	sync->switch_state = true;
+	sync->control_ids = calloc(count, sizeof(sync->control_ids[0]));
+	if (sync->control_ids == NULL)
+		return -ENOMEM;
+	snd_config_for_each(i, next, conf) {
+		snd_config_t *n = snd_config_iterator_entry(i);
+		if (snd_config_get_string(n, &str) < 0) {
+			SNDERR("strings are expected in sync array");
+			return -EINVAL;
+		}
+		eid = &sync->control_ids[index];
+		snd_ctl_elem_id_clear(eid);
+		err = snd_ctl_ascii_elem_id_parse(eid, str);
+		if (err < 0) {
+			SNDERR("unable to parse control id '%s'!", str);
+			return -EINVAL;
+		}
+		sync->control_items++;
+		index++;
+	}
+
+	return 0;
+}
+
+static int parse_sync_compound(snd_ctl_remap_t *priv, snd_config_t *conf)
+{
+	snd_config_iterator_t i, next;
+	snd_ctl_elem_id_t eid;
+	snd_ctl_numid_t *numid;
+	bool eid_found = false;
+	bool controls_found = false;
+	int count, err;
+
+	snd_ctl_elem_id_clear(&eid);
+	snd_config_for_each(i, next, conf) {
+		snd_config_t *n = snd_config_iterator_entry(i);
+		const char *id, *str;
+		if (snd_config_get_id(n, &id) < 0)
+			continue;
+		if (strcmp(id, "switch") == 0) {
+			if (snd_config_get_string(n, &str) < 0) {
+				SNDERR("String is expected for switch");
+				return -EINVAL;
+			}
+			err = snd_ctl_ascii_elem_id_parse(&eid, str);
+			if (err < 0) {
+				SNDERR("unable to parse id '%s'!", str);
+				return -EINVAL;
+			}
+			eid_found = true;
+		}
+		if (strcmp(id, "controls") == 0) {
+			count = snd_config_is_array(n);
+			if (count <= 0) {
+				SNDERR("Array is expected for sync!");
+				return -EINVAL;
+			}
+			err = parse_sync1(priv, count, n);
+			if (err < 0)
+				return err;
+			controls_found = true;
+		}
+	}
+
+	if (!controls_found || !eid_found)
+		return 0;
+
+	numid = remap_numid_new(priv, 0, ++priv->numid_app_last);
+	if (numid == NULL)
+		return -ENOMEM;
+	eid.numid = numid->numid_app;
+	priv->sync[priv->sync_items - 1].switch_id = eid;
+	priv->sync_switch_items++;
+	return 0;
+}
+
+static int parse_sync(snd_ctl_remap_t *priv, snd_config_t *conf)
+{
+	snd_config_iterator_t i, next;
+	int count, err;
+
+	if (conf == NULL)
+		return 0;
+	snd_config_for_each(i, next, conf) {
+		snd_config_t *n = snd_config_iterator_entry(i);
+		if (snd_config_get_type(n) == SND_CONFIG_TYPE_COMPOUND) {
+			err = parse_sync_compound(priv, n);
+		} else {
+			count = snd_config_is_array(n);
+			if (count <= 0) {
+				SNDERR("Array is expected for sync!");
+				return -EINVAL;
+			}
+			err = parse_sync1(priv, count, n);
+		}
+		if (err < 0)
+			return err;
+	}
+
+	return 0;
+}
+
 /**
- * \brief Creates a new remap & map control handle
+ * \brief Creates a new remap/map/sync control handle
  * \param handlep Returns created control handle
  * \param name Name of control device
  * \param remap Remap configuration
  * \param map Map configuration
+ * \param sync Sync configuration
  * \param child child configuration root
  * \param mode Control handle mode
  * \retval zero on success otherwise a negative error code
@@ -1154,14 +1618,15 @@ static int parse_map(snd_ctl_remap_t *priv, snd_config_t *conf)
  *          changed in future.
  */
 int snd_ctl_remap_open(snd_ctl_t **handlep, const char *name, snd_config_t *remap,
-		       snd_config_t *map, snd_ctl_t *child, int mode)
+		       snd_config_t *map, snd_config_t *sync, snd_ctl_t *child, int mode)
 {
 	snd_ctl_remap_t *priv;
 	snd_ctl_t *ctl;
+	size_t index;
 	int result, err;
 
 	/* no-op, remove the plugin */
-	if (!remap && !map)
+	if (!remap && !map && !sync)
 		goto _noop;
 
 	priv = calloc(1, sizeof(*priv));
@@ -1180,8 +1645,14 @@ int snd_ctl_remap_open(snd_ctl_t **handlep, const char *name, snd_config_t *rema
 		goto _err;
 	}
 
+	err = parse_sync(priv, sync);
+	if (err < 0) {
+		result = err;
+		goto _err;
+	}
+
 	/* no-op check, remove the plugin */
-	if (priv->map_items == 0 && priv->remap_items == 0) {
+	if (priv->map_items == 0 && priv->remap_items == 0 && priv->sync_items == 0) {
 		remap_free(priv);
  _noop:
 		free(child->name);
@@ -1192,13 +1663,16 @@ int snd_ctl_remap_open(snd_ctl_t **handlep, const char *name, snd_config_t *rema
 		return 0;
 	}
 
-	priv->map_read_queue = calloc(priv->map_items, sizeof(priv->map_read_queue[0]));
-	if (priv->map_read_queue == NULL) {
+	priv->event_items = priv->map_items;
+	for (index = 0; index < priv->sync_items; index++)
+		priv->event_items += priv->sync[index].control_items;
+	priv->event_queue = calloc(priv->event_items, sizeof(priv->event_queue[0]));
+	if (priv->event_queue == NULL) {
 		result = -ENOMEM;
 		goto _err;
 	}
 
-	priv->numid_remap_active = priv->map_items > 0;
+	priv->numid_remap_active = priv->map_items > 0 || priv->sync_items;
 
 	priv->child = child;
 	err = snd_ctl_new(&ctl, SND_CTL_TYPE_REMAP, name, mode);
@@ -1264,6 +1738,27 @@ ctl.name {
 				vindex.0 1	# stereo to mono (second channel)
 			}
 		}
+		# join two stereo to one stereo (minimum value is returned for read operation)
+		CREATE_ID4_STR {
+			SRC_ID5_STR.vindex.0 [ 0 1 ] # source channels 0+1 to merged channel 0
+			SRC_ID6_STR.vindex.1 [ 0 1 ] # source channels 0+1 to merged channel 1
+		}
+	}
+	sync {
+		# synchronize multiple controls without any translations
+		sample_group_1 [
+			SYNC_ID1_STR
+			SYNC_ID2_STR
+		]
+		# synchronize multiple controls without any translations
+		# add functionality on/off switch
+		sample_group_2 {
+			switch SYNC_SWITCH_ID
+			controls [
+				SYNC_ID3_STR
+				SYNC_ID4_STR
+			]
+		}
 	}
 }
 \endcode
@@ -1295,6 +1790,7 @@ int _snd_ctl_remap_open(snd_ctl_t **handlep, char *name, snd_config_t *root, snd
 	snd_config_t *child = NULL;
 	snd_config_t *remap = NULL;
 	snd_config_t *map = NULL;
+	snd_config_t *sync = NULL;
 	snd_ctl_t *cctl;
 	int err;
 
@@ -1313,6 +1809,10 @@ int _snd_ctl_remap_open(snd_ctl_t **handlep, char *name, snd_config_t *root, snd
 			map = n;
 			continue;
 		}
+		if (strcmp(id, "sync") == 0) {
+			sync = n;
+			continue;
+		}
 		if (strcmp(id, "child") == 0) {
 			child = n;
 			continue;
@@ -1327,7 +1827,7 @@ int _snd_ctl_remap_open(snd_ctl_t **handlep, char *name, snd_config_t *root, snd
 	err = _snd_ctl_open_child(&cctl, root, child, mode, conf);
 	if (err < 0)
 		return err;
-	err = snd_ctl_remap_open(handlep, name, remap, map, cctl, mode);
+	err = snd_ctl_remap_open(handlep, name, remap, map, sync, cctl, mode);
 	if (err < 0)
 		snd_ctl_close(cctl);
 	return err;
