@@ -63,7 +63,7 @@ use case verbs for that sound card. i.e.:
 # Example master file for blah sound card
 # By Joe Blogs <joe@bloggs.org>
 
-Syntax 6
+Syntax 7
 
 # Use Case name for user interface
 Comment "Nice Abstracted Soundcard"
@@ -376,7 +376,9 @@ Evaluation order   | Configuration block | Evaluation restart
 ------------------:|---------------------|--------------------
 1                  | Define              | No
 2                  | Include             | Yes
-3                  | If                  | Yes
+3                  | Variant             | Yes
+4                  | Macro               | Yes
+5                  | If                  | Yes
 
 
 ### Substitutions
@@ -384,26 +386,58 @@ Evaluation order   | Configuration block | Evaluation restart
 The dynamic tree identifiers and assigned values in the configuration tree are
 substituted. The substitutes strings are in the table bellow.
 
-Substituted string   | Value
----------------------|---------------------
-${OpenName}          | Original UCM card name (passed to snd_use_case_mgr_open())
-${ConfLibDir}        | Library top-level configuration directory (e.g. /usr/share/alsa)
-${ConfTopDir}        | Top-level UCM configuration directory (e.g. /usr/share/alsa/ucm2)
-${ConfDir}           | Card's UCM configuration directory (e.g. /usr/share/alsa/ucm2/conf.d/USB-Audio)
-${ConfName}          | Configuration name (e.g. USB-Audio.conf)
-${CardNumber}        | Real ALSA card number (or empty string for the virtual UCM card)
-${CardId}            | ALSA card identifier (see snd_ctl_card_info_get_id())
-${CardDriver}        | ALSA card driver (see snd_ctl_card_info_get_driver())
-${CardName}          | ALSA card name (see snd_ctl_card_info_get_name())
-${CardLongName}      | ALSA card long name (see snd_ctl_card_info_get_longname())
-${CardComponents}    | ALSA card components (see snd_ctl_card_info_get_components())
+Substituted string     | Value
+-----------------------|---------------------
+${LibCaps}             | Library capabilities (string like '*a*b*c*') [**Syntax 8**]
+${OpenName}            | Original UCM card name (passed to snd_use_case_mgr_open())
+${ConfLibDir}          | Library top-level configuration directory (e.g. /usr/share/alsa)
+${ConfTopDir}          | Top-level UCM configuration directory (e.g. /usr/share/alsa/ucm2)
+${ConfDir}             | Card's UCM configuration directory (e.g. /usr/share/alsa/ucm2/conf.d/USB-Audio)
+${ConfName}            | Configuration name (e.g. USB-Audio.conf)
+${CardNumber}          | Real ALSA card number (or empty string for the virtual UCM card)
+${CardId}              | ALSA card identifier (see snd_ctl_card_info_get_id())
+${CardDriver}          | ALSA card driver (see snd_ctl_card_info_get_driver())
+${CardName}            | ALSA card name (see snd_ctl_card_info_get_name())
+${CardLongName}        | ALSA card long name (see snd_ctl_card_info_get_longname())
+${CardComponents}      | ALSA card components (see snd_ctl_card_info_get_components())
 ${env:\<str\>}         | Environment variable \<str\>
 ${sys:\<str\>}         | Contents of sysfs file \<str\>
+${sys-card:\<str\>}    | Contents of sysfs file in /sys/class/sound/card? tree [**Syntax 8**]
 ${var:\<str\>}         | UCM parser variable (set using a _Define_ block)
 ${eval:\<str\>}        | Evaluate expression like *($var+2)/3* [**Syntax 5**]
 ${find-card:\<str\>}   | Find a card - see _Find card substitution_ section
 ${find-device:\<str\>} | Find a device - see _Find device substitution_ section
 
+General note: If two dollars '$$' instead one dolar '$' are used for the
+substitution identification, the error is ignored (e.g. file does not
+exists in sysfs tree).
+
+Note for *var* substitution: If the first characters is minus ('-') the
+empty string is substituted when the variable is not defined.
+
+Note for *sys* and *sys-card* substitutions: since syntax 8, there is
+also extension to fetch data from given range with the optional conversion
+to hexadecimal format when the source file has binary contents.
+
+Example - fetch bytes from positions 0x10..0x15 (6 bytes):
+
+~~~{.html}
+Define.Bytes1 "${sys-card:[type=hex,pos=0x10,size=6]device/../descriptors}"
+~~~
+
+Example - fetch one byte from position 0x22:
+
+~~~{.html}
+Define.Bytes2 "${sys-card:[type=hex,pos=0x22]device/../descriptors}"
+~~~
+
+Replace *type=hex* with *type=ascii* or omit this variable settings to work with ASCII characters.
+
+
+#### Library capabilities
+
+None at the moment. The list will grow after *Syntax 8* (library 1.2.14).
+
 #### Special whole string substitution
 
 Substituted string   | Value
@@ -489,7 +523,8 @@ DefineMacro.macro1 {
 The arguments in the macro are refered as the variables with the double
 underscore name prefix (like *__variable*). The configuration block in
 the DefineMacro subtree is always evaluated (including arguments and variables)
-at the time of the instantiation.
+at the time of the instantiation. Argument string substitutions
+(for multiple macro call levels) were added in *Syntax* version *7*.
 
 The macros can be instantiated (expanded) using:
 
@@ -563,6 +598,15 @@ Field                | Description
 String               | string
 Regex                | regex expression (extended posix, ignore case)
 
+#### Path is present (Type Path)
+
+Field                | Description
+---------------------|-----------------------
+Path                 | path (filename)
+Mode                 | exist,read,write,exec
+
+Note: Substitution for Path and Mode fields were added in *Syntax* version *7*.
+
 #### ALSA control element exists (Type ControlExists)
 
 Field                | Description
