@@ -20,7 +20,7 @@
  *  transition sequences, multiple client access and user defined use
  *  cases was kindly sponsored by Wolfson Microelectronics PLC.
  *
- *  Copyright (C) 2019 Red Hat Inc.
+ *  Copyright (C) 2019-2025 Red Hat Inc.
  *  Authors: Jaroslav Kysela <perex@perex.cz>
  */
 
@@ -30,6 +30,19 @@
 #include <limits.h>
 #include <regex.h>
 
+/* capabilities string like "*a*b*c" will be used to check library extensions */
+/* use Needle like "*a*" or regex expression for a match */
+#define LIB_CAPS_STRING "*" "*"
+
+static unsigned char _hex_table[16] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
+
+static char *rval_lib_caps(snd_use_case_mgr_t *uc_mgr ATTRIBUTE_UNUSED)
+{
+	if (uc_mgr->conf_format < 8)
+		return NULL;
+	return strdup(LIB_CAPS_STRING);
+}
+
 static char *rval_open_name(snd_use_case_mgr_t *uc_mgr)
 {
 	const char *name;
@@ -191,6 +204,7 @@ static char *rval_card_id_by_name(snd_use_case_mgr_t *uc_mgr, const char *id)
 	return strdup(snd_ctl_card_info_get_id(ctl_list->ctl_info));
 }
 
+#ifndef DOC_HIDDEN
 typedef struct lookup_iterate *(*lookup_iter_fcn_t)
 			(snd_use_case_mgr_t *uc_mgr, struct lookup_iterate *iter);
 typedef const char *(*lookup_fcn_t)(void *);
@@ -212,6 +226,7 @@ struct lookup_iterate {
 	struct ctl_list *ctl_list;
 	void *info;
 };
+#endif /* DOC_HIDDEN */
 
 static char *rval_lookup_main(snd_use_case_mgr_t *uc_mgr,
 			      const char *query,
@@ -490,27 +505,130 @@ static char *rval_env(snd_use_case_mgr_t *uc_mgr ATTRIBUTE_UNUSED, const char *i
 {
 	char *e;
 
-	e = getenv(id);
+	if (*id == '-') {
+		e = getenv(id + 1);
+		if (e == NULL)
+			e = "";
+	} else {
+		e = getenv(id);
+	}
 	if (e)
 		return strdup(e);
 	return NULL;
 }
 
-static char *rval_sysfs(snd_use_case_mgr_t *uc_mgr ATTRIBUTE_UNUSED, const char *id)
+#define RANGE_TYPE_ASCII 0
+#define RANGE_TYPE_HEX   1
+
+static int parse_position(snd_config_t *config, const char *name, ssize_t *pos, bool optional)
+{
+	snd_config_t *d;
+	const char *s;
+	long v;
+
+	if (snd_config_search(config, name, &d)) {
+		if (optional) {
+			*pos = -1;
+			return 0;
+		}
+		uc_error("Unable to find field '%s'", name);
+		return -1;
+	}
+	if (!snd_config_get_integer(d, &v))
+		goto fin;
+	if (snd_config_get_string(d, &s))
+		return -1;
+	if (safe_strtol(s, &v)) {
+		uc_error("Unable to parse position '%s'", s);
+		return -1;
+	}
+fin:
+	*pos = v;
+	return 0;
+}
+
+static int parse_range(const char *cfg, int *type, ssize_t *pos, ssize_t *size)
+{
+	snd_config_t *config, *d;
+	int err, retval = 0;
+	const char *s;
+
+	err = snd_config_load_string(&config, cfg, 0);
+	if (err < 0) {
+		uc_error("The range arguments '%s' are invalid", cfg);
+		return -1;
+	}
+	if (snd_config_search(config, "type", &d)) {
+		*type = RANGE_TYPE_ASCII;
+	} else {
+		if (snd_config_get_string(d, &s))
+			goto null;
+		if (strcasecmp(s, "ascii") == 0) {
+			*type = RANGE_TYPE_ASCII;
+		} else if (strcasecmp(s, "hex") == 0) {
+			*type = RANGE_TYPE_HEX;
+		} else {
+			uc_error("Unknown range type '%s'", s);
+		}
+	}
+	*pos = 0;
+	*size = -1;
+	if (parse_position(config, "pos", pos, false) ||
+	    parse_position(config, "size", size, true)) {
+		retval = -1;
+		goto null;
+	}
+
+	if (*size <= 0)
+		*size = 1;
+	if (*pos < 0) {
+		uc_error("Invalid start position");
+		retval = -1;
+		goto null;
+	}
+
+null:
+	snd_config_delete(config);
+	return retval;
+}
+
+static char *rval_sysfs_main(snd_use_case_mgr_t *uc_mgr, const char *top_path, const char *id)
 {
 	char path[PATH_MAX], link[PATH_MAX + 1];
-	struct stat sb;
-	ssize_t len;
-	const char *e;
-	int fd;
+	struct stat64 sb;
+	ssize_t len, range_start = -1, range_size = -1;
+	const char *e, *s;
+	int fd, type = RANGE_TYPE_ASCII;
 
 	e = uc_mgr_sysfs_root();
 	if (e == NULL)
 		return NULL;
+	if (id[0] == '[') {
+		if (uc_mgr->conf_format < 8) {
+			uc_error("Sysfs ranges are supported in v8+ syntax");
+			return NULL;
+		}
+		s = strchr(id, ']');
+		if (s == NULL)
+			return NULL;
+		len = s - id - 1;
+		if ((size_t)(len - 1) > sizeof(link) - 1)
+			return NULL;
+		strncpy(link, id + 1, len);
+		link[len] = '\0';
+		if (parse_range(link, &type, &range_start, &range_size)) {
+			uc_error("sysfs: cannot parse hex range '%s'", link);
+			return NULL;
+		}
+		id = s + 1;
+	}
 	if (id[0] == '/')
 		id++;
-	snprintf(path, sizeof(path), "%s/%s", e, id);
-	if (lstat(path, &sb) != 0)
+	if (top_path)
+		snprintf(path, sizeof(path), "%s/%s/%s", e, top_path, id);
+	else
+		snprintf(path, sizeof(path), "%s/%s", e, id);
+	if (lstat64(path, &sb) != 0)
 		return NULL;
 	if (S_ISLNK(sb.st_mode)) {
 		len = readlink(path, link, sizeof(link) - 1);
@@ -534,28 +652,83 @@ static char *rval_sysfs(snd_use_case_mgr_t *uc_mgr ATTRIBUTE_UNUSED, const char
 		uc_error("sysfs open failed for '%s' (%d)", path, errno);
 		return NULL;
 	}
-	len = read(fd, path, sizeof(path)-1);
+	len = sizeof(path) - 1;
+	if (range_start > 0 && lseek(fd, range_start, SEEK_SET) != range_start) {
+		uc_error("sysfs seek failed (%d)", errno);
+		close(fd);
+		return NULL;
+	}
+	if (range_size > 0) {
+		if (range_size > len) {
+			uc_error("sysfs EOB for '%s'", path);
+			close(fd);
+			return NULL;
+		} else {
+			len = range_size;
+		}
+	}
+	len = read(fd, path, len);
 	close(fd);
 	if (len < 0) {
 		uc_error("sysfs unable to read value '%s' (%d)", path, errno);
 		return NULL;
 	}
+	if (type == RANGE_TYPE_HEX && range_start >= 0) {
+		char *m = malloc(len * 2 + 1);
+		ssize_t idx;
+		if (m == NULL)
+			return NULL;
+		for (idx = 0; idx < len; idx++) {
+			m[(idx * 2) + 0] = _hex_table[((unsigned char)path[idx]) >> 4];
+			m[(idx * 2) + 1] = _hex_table[((unsigned char)path[idx]) & 0x0f];
+		}
+		m[len * 2] = '\0';
+		return m;
+	}
 	while (len > 0 && path[len-1] == '\n')
 		len--;
 	path[len] = '\0';
 	return strdup(path);
 }
 
+static char *rval_sysfs(snd_use_case_mgr_t *uc_mgr, const char *id)
+{
+	return rval_sysfs_main(uc_mgr, NULL, id);
+}
+
+static char *rval_sysfs_card(snd_use_case_mgr_t *uc_mgr, const char *id)
+{
+	char top_path[32], *s;
+
+	if (uc_mgr->conf_format < 8) {
+		uc_error("sys-card is supported in v8+ syntax");
+		return NULL;
+	}
+	s = get_card_number(uc_mgr_get_master_ctl(uc_mgr));
+	snprintf(top_path, sizeof(top_path), "class/sound/card%s", s);
+	free(s);
+	return rval_sysfs_main(uc_mgr, top_path, id);
+}
+
 static char *rval_var(snd_use_case_mgr_t *uc_mgr, const char *id)
 {
 	const char *v;
+	bool ignore_not_found = false;
 
 	if (uc_mgr->conf_format < 3) {
 		uc_error("variable substitution is supported in v3+ syntax");
 		return NULL;
 	}
 
+	if (id[0] == '-') {
+		ignore_not_found = true;
+		id++;
+	} else if (id[0] == '@') {
+		ignore_not_found = true;
+	}
 	v = uc_mgr_get_variable(uc_mgr, id);
+	if (v == NULL && ignore_not_found)
+		v = "";
 	if (v)
 		return strdup(v);
 	return NULL;
@@ -582,7 +755,7 @@ static char *rval_eval(snd_use_case_mgr_t *uc_mgr, const char *e)
 	int err;
 
 	if (uc_mgr->conf_format < 5) {
-		uc_error("variable substitution is supported in v5+ syntax");
+		uc_error("variable evaluation is supported in v5+ syntax");
 		return NULL;
 	}
 	err = _snd_eval_string(&dst, e, rval_eval_var_cb, uc_mgr);
@@ -597,6 +770,41 @@ static char *rval_eval(snd_use_case_mgr_t *uc_mgr, const char *e)
 	return r;
 }
 
+static int rval_evali(snd_use_case_mgr_t *uc_mgr, snd_config_t *node, const char *e)
+{
+	snd_config_t *dst;
+	const char *id;
+	char *s;
+	size_t l;
+	int err;
+
+	if (uc_mgr->conf_format < 6) {
+		uc_error("variable evaluation is supported in v6+ syntax");
+		return -EINVAL;
+	}
+	err = snd_config_get_id(node, &id);
+	if (err < 0)
+		return err;
+	l = strlen(e);
+	if (e[l-1] != '}')
+		return -EINVAL;
+	s = malloc(l + 1);
+	if (s == NULL)
+		return -ENOMEM;
+	strcpy(s, e);
+	s[l-1] = '\0';
+	err = _snd_eval_string(&dst, s + 8, rval_eval_var_cb, uc_mgr);
+	free(s);
+	if (err < 0) {
+		uc_error("unable to evaluate '%s'", e);
+		return err;
+	}
+	err = snd_config_set_id(dst, id);
+	if (err < 0)
+		return err;
+	return snd_config_substitute(node, dst);
+}
+
 #define MATCH_VARIABLE(name, id, fcn, empty_ok)				\
 	if (strncmp((name), (id), sizeof(id) - 1) == 0) { 		\
 		rval = fcn(uc_mgr);					\
@@ -686,6 +894,7 @@ __std:
 			goto __std;
 		}
 		fcn2 = NULL;
+		MATCH_VARIABLE(value, "${LibCaps}", rval_lib_caps, false);
 		MATCH_VARIABLE(value, "${OpenName}", rval_open_name, false);
 		MATCH_VARIABLE(value, "${ConfLibDir}", rval_conf_libdir, false);
 		MATCH_VARIABLE(value, "${ConfTopDir}", rval_conf_topdir, false);
@@ -699,6 +908,7 @@ __std:
 		MATCH_VARIABLE(value, "${CardComponents}", rval_card_components, true);
 		MATCH_VARIABLE2(value, "${env:", rval_env, false);
 		MATCH_VARIABLE2(value, "${sys:", rval_sysfs, false);
+		MATCH_VARIABLE2(value, "${sys-card:", rval_sysfs_card, false);
 		MATCH_VARIABLE2(value, "${var:", rval_var, true);
 		MATCH_VARIABLE2(value, "${eval:", rval_eval, false);
 		MATCH_VARIABLE2(value, "${find-card:", rval_card_lookup, false);
@@ -727,6 +937,8 @@ __match2:
 			strncpy_with_escape(v2, value + idsize, rvalsize);
 			idsize += rvalsize + 1;
 			if (*v2 == '$' && uc_mgr->conf_format >= 3) {
+				if (strncmp(value, "${eval:", 7) == 0)
+					goto __direct_fcn2;
 				tmp = uc_mgr_get_variable(uc_mgr, v2 + 1);
 				if (tmp == NULL) {
 					uc_error("define '%s' is not reachable in this context!", v2 + 1);
@@ -735,6 +947,7 @@ __match2:
 					rval = fcn2(uc_mgr, tmp);
 				}
 			} else {
+__direct_fcn2:
 				rval = fcn2(uc_mgr, v2);
 			}
 			goto __rval;
@@ -816,6 +1029,8 @@ int uc_mgr_substitute_tree(snd_use_case_mgr_t *uc_mgr, snd_config_t *node)
 				return err;
 			if (!uc_mgr_substitute_check(s2))
 				return 0;
+			if (strncmp(s2, "${evali:", 8) == 0)
+				return rval_evali(uc_mgr, node, s2);
 			err = uc_mgr_get_substituted_value(uc_mgr, &s, s2);
 			if (err < 0)
 				return err;
@@ -826,6 +1041,12 @@ int uc_mgr_substitute_tree(snd_use_case_mgr_t *uc_mgr, snd_config_t *node)
 		}
 		return 0;
 	}
+	/* exception - macros are evaluated when instantied */
+	err = snd_config_get_id(node, &id);
+	if (err < 0)
+		return err;
+	if (id && strcmp(id, "DefineMacro") == 0)
+		return 0;
 	snd_config_for_each(i, next, node) {
 		n = snd_config_iterator_entry(i);
 		err = uc_mgr_substitute_tree(uc_mgr, n);
