@@ -1,7 +1,7 @@
 /*
  *  Old PCM API compilation test
  *
- *     Author: Jaroslav Kysela <perex@suse.cz>
+ *     Author: Jaroslav Kysela <perex@perex.cz>
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -31,7 +31,7 @@
 #include "../include/asoundlib.h"
 #include <sys/time.h>
 
-typedef void (myfcn)(void *);
+typedef int (myfcn)(const snd_pcm_hw_params_t *);
 
 int main(int argc ATTRIBUTE_UNUSED, char *argv[] ATTRIBUTE_UNUSED)
 {
